@@ -1,32 +1,31 @@
 /* action.h
  * Header file for the action object
  *
- * File begun on 2007-08-06 by RGerhards (extracted from syslogd.c)
+ * File begun on 2007-08-06 by RGerhards (extracted from syslogd.c, which
+ * was under BSD license at the time of rsyslog fork)
  *
- * Copyright 2007-2011 Rainer Gerhards and Adiscon GmbH.
+ * Copyright 2007-2012 Adiscon GmbH.
  *
  * This file is part of rsyslog.
  *
- * Rsyslog is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
- *
- * Rsyslog is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with Rsyslog.  If not, see <http://www.gnu.org/licenses/>.
- *
- * A copy of the GPL can be found in the file "COPYING" in this distribution.
+ * Licensed under the Apache License, Version 2.0 (the "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
+ * 
+ *       http://www.apache.org/licenses/LICENSE-2.0
+ *       -or-
+ *       see COPYING.ASL20 in the source distribution
+ * 
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
  */
 #ifndef	ACTION_H_INCLUDED
 #define	ACTION_H_INCLUDED 1
 
 #include "syslogd-types.h"
-#include "sync.h"
 #include "queue.h"
 
 /* external data - this is to be removed when we change the action
@@ -85,10 +84,14 @@ struct action_s {
 				 * processed - it is also used to detect duplicates.
 				 */
 	qqueue_t *pQueue;	/* action queue */
-	SYNC_OBJ_TOOL;		/* required for mutex support */
+	pthread_mutex_t mutAction; /* primary action mutex */
 	pthread_mutex_t mutActExec; /* mutex to guard actual execution of doAction for single-threaded modules */
 	uchar *pszName;		/* action name (for documentation) */
 	DEF_ATOMIC_HELPER_MUT(mutCAS);
+	/* for statistics subsystem */
+	statsobj_t *statsobj;
+	STATSCOUNTER_DEF(ctrProcessed, mutCtrProcessed);
+	STATSCOUNTER_DEF(ctrFail, mutCtrFail);
 };
 
 
@@ -104,8 +107,6 @@ rsRetVal actionWriteToAction(action_t *pAction);
 rsRetVal actionCallHUPHdlr(action_t *pAction);
 rsRetVal actionClassInit(void);
 rsRetVal addAction(action_t **ppAction, modInfo_t *pMod, void *pModData, omodStringRequest_t *pOMSR, struct cnfparamvals *actParams, struct cnfparamvals *queueParams, int bSuspended);
-rsRetVal actionNewScope(void);
-rsRetVal actionRestoreScope(void);
 rsRetVal activateActions(void);
 rsRetVal actionNewInst(struct nvlst *lst, action_t **ppAction);
 
