@@ -292,7 +292,6 @@ doGetQueueType(struct nvlst *valnode, struct cnfparamdescr *param,
 			      param->name, cstr);
 		free(cstr);
 	}
-dbgprintf("XXXXX: queue type: %d\n", (int)val->val.d.n);
 	val->val.datatype = 'N';
 }
 
@@ -437,8 +436,8 @@ nvlstGetParam(struct nvlst *valnode, struct cnfparamdescr *param,
 {
 	uchar *cstr;
 
-	dbgprintf("XXXX: in nvlstGetParam, name '%s', type %d\n",
-		  param->name, (int) param->type);
+	dbgprintf("XXXX: in nvlstGetParam, name '%s', type %d, valnode->bUsed %d\n",
+		  param->name, (int) param->type, valnode->bUsed);
 	valnode->bUsed = 1;
 	val->bUsed = 1;
 	switch(param->type) {
@@ -624,14 +623,13 @@ cnfactlstAddSysline(struct cnfactlst* actlst, char *line)
 	struct cnfcfsyslinelst *cflst;
 
 	if((cflst = malloc(sizeof(struct cnfcfsyslinelst))) != NULL)   {
-		cflst->next = NULL;
 		cflst->line = line;
 		if(actlst->syslines == NULL) {
-			actlst->syslines = cflst;
+			cflst->next = NULL;
 		} else {
 			cflst->next = actlst->syslines;
-			actlst->syslines = cflst;
 		}
+		actlst->syslines = cflst;
 	}
 	return actlst;
 }
@@ -744,9 +742,11 @@ var2Number(struct var *r, int *bSuccess)
 	if(r->datatype == 'S') {
 		n = es_str2num(r->d.estr, bSuccess);
 	} else {
-		*bSuccess = 1;
+		n = r->d.n;
+		if(bSuccess)
+			*bSuccess = 1;
 	}
-	return r->d.n;
+	return n;
 }
 
 /* ensure that retval is a string; if string is no number,
@@ -1129,8 +1129,9 @@ cnfexprEval(struct cnfexpr *expr, struct var *ret, void* usrptr)
 				ret->d.n = 1ll;
 			else 
 				ret->d.n = 0ll;
+			if(r.datatype == 'S') es_deleteStr(r.d.estr); 
 		}
-		FREE_BOTH_RET;
+		if(l.datatype == 'S') es_deleteStr(l.d.estr);
 		break;
 	case AND:
 		cnfexprEval(expr->l, &l, usrptr);
@@ -1141,10 +1142,11 @@ cnfexprEval(struct cnfexpr *expr, struct var *ret, void* usrptr)
 				ret->d.n = 1ll;
 			else 
 				ret->d.n = 0ll;
+			if(r.datatype == 'S') es_deleteStr(r.d.estr); 
 		} else {
 			ret->d.n = 0ll;
 		}
-		FREE_BOTH_RET;
+		if(l.datatype == 'S') es_deleteStr(l.d.estr);
 		break;
 	case NOT:
 		cnfexprEval(expr->r, &r, usrptr);
@@ -1409,6 +1411,9 @@ cnfrulePrint(struct cnfrule *rule)
 	dbgprintf("------ end rule %p\n", rule);
 }
 
+/* note: the sysline itself was already freed during processing
+ * and as such MUST NOT be freed again!
+ */
 void
 cnfcfsyslinelstDestruct(struct cnfcfsyslinelst *cfslst)
 {
@@ -1416,7 +1421,6 @@ cnfcfsyslinelstDestruct(struct cnfcfsyslinelst *cfslst)
 	while(cfslst != NULL) {
 		toDel = cfslst;
 		cfslst = cfslst->next;
-		free(toDel->line);
 		free(toDel);
 	}
 }
