@@ -21,25 +21,24 @@
  * To test under Linux:
  * echo test1 > /dev/kmsg
  *
- * Copyright (C) 2008-2011 by Rainer Gerhards and Adiscon GmbH
+ * Copyright (C) 2008-2012 Adiscon GmbH
  *
  * This file is part of rsyslog.
  *
- * Rsyslog is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
- *
- * Rsyslog is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with Rsyslog.  If not, see <http://www.gnu.org/licenses/>.
- *
- * A copy of the GPL can be found in the file "COPYING" in this distribution.
-*/
+ * Licensed under the Apache License, Version 2.0 (the "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
+ * 
+ *       http://www.apache.org/licenses/LICENSE-2.0
+ *       -or-
+ *       see COPYING.ASL20 in the source distribution
+ * 
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
+ */
 #include "config.h"
 #include "rsyslog.h"
 #include <stdio.h>
@@ -48,6 +47,7 @@
 #include <stdarg.h>
 #include <ctype.h>
 #include <stdlib.h>
+#include <sys/socket.h>
 
 #include "dirty.h"
 #include "cfsysline.h"
@@ -56,6 +56,7 @@
 #include "module-template.h"
 #include "datetime.h"
 #include "imklog.h"
+#include "net.h"
 #include "glbl.h"
 #include "prop.h"
 #include "unicode-helper.h"
@@ -69,6 +70,7 @@ DEF_IMOD_STATIC_DATA
 DEFobjCurrIf(datetime)
 DEFobjCurrIf(glbl)
 DEFobjCurrIf(prop)
+DEFobjCurrIf(net)
 
 /* config settings */
 typedef struct configSettings_s {
@@ -80,7 +82,6 @@ typedef struct configSettings_s {
 	int iFacilIntMsg; /* the facility to use for internal messages (set by driver) */
 	uchar *pszPath;
 	int console_log_level;
-	char *symfile; /* TODO: actually unsued currently! */
 } configSettings_t;
 static configSettings_t cs;
 
@@ -102,7 +103,6 @@ initConfigSettings(void)
 	cs.bPermitNonKernel = 0;
 	cs.console_log_level = -1;
 	cs.pszPath = NULL;
-	cs.symfile = NULL; 
 	cs.iFacilIntMsg = klogFacilIntMsg();
 }
 
@@ -136,8 +136,8 @@ enqMsg(uchar *msg, uchar* pszTag, int iFacility, int iSeverity, struct timeval *
 	MsgSetRcvFromIP(pMsg, pLocalHostIP);
 	MsgSetHOSTNAME(pMsg, glbl.GetLocalHostName(), ustrlen(glbl.GetLocalHostName()));
 	MsgSetTAG(pMsg, pszTag, ustrlen(pszTag));
-	pMsg->iFacility = LOG_FAC(iFacility);
-	pMsg->iSeverity = LOG_PRI(iSeverity);
+	pMsg->iFacility = iFacility;
+	pMsg->iSeverity = iSeverity;
 	CHKiRet(submitMsg(pMsg));
 
 finalize_it:
@@ -192,11 +192,9 @@ rsRetVal imklogLogIntMsg(int priority, char *fmt, ...)
 	DEFiRet;
 	va_list ap;
 	uchar msgBuf[2048]; /* we use the same size as sysklogd to remain compatible */
-	uchar *pLogMsg;
 
 	va_start(ap, fmt);
 	vsnprintf((char*)msgBuf, sizeof(msgBuf) / sizeof(char), fmt, ap);
-	pLogMsg = msgBuf;
 	va_end(ap);
 
 	logmsgInternal(NO_ERRCODE ,priority, msgBuf, 0);
@@ -216,15 +214,14 @@ rsRetVal Syslog(int priority, uchar *pMsg, struct timeval *tp)
 	DEFiRet;
 
 	/* then check if we have two PRIs. This can happen in case of systemd,
-	 * in which case the second PRI is the rigth one.
-	 * TODO: added kernel timestamp support to this PoC. -- rgerhards, 2011-03-18
+	 * in which case the second PRI is the right one.
 	 */
-	if(pMsg[3] == '<') { /* could be a pri... */
-		uchar *pMsgTmp = pMsg + 3;
+	if(pMsg[3] == '<' || (pMsg[3] == ' ' && pMsg[4] == '<')) { /* could be a pri... */
+		uchar *pMsgTmp = pMsg + ((pMsg[3] == '<') ? 3 : 4);
 		localRet = parsePRI(&pMsgTmp, &pri);
 		if(localRet == RS_RET_OK && pri >= 8 && pri <= 192) {
 			/* *this* is our PRI */
-			DBGPRINTF("imklog detected secondary PRI in klog msg\n");
+			DBGPRINTF("imklog detected secondary PRI(%d) in klog msg\n", pri);
 			pMsg = pMsgTmp;
 			priority = pri;
 		}
@@ -301,21 +298,8 @@ CODESTARTendCnfLoad
 		loadModConf->pszPath = cs.pszPath;
 	}
 	cs.pszPath = NULL;
-	if((cs.symfile == NULL) || (cs.symfile[0] == '\0')) {
-		loadModConf->symfile = NULL;
-		if(cs.symfile != NULL)
-			free(cs.symfile);
-	} else {
-		loadModConf->symfile = cs.symfile;
-	}
-	cs.symfile = NULL;
 
 	loadModConf = NULL; /* done loading */
-	/* free legacy config vars */
-	free(cs.pszPath);
-	cs.pszPath = NULL;
-	free(cs.symfile);
-	cs.symfile = NULL;
 ENDendCnfLoad
 
 
@@ -361,6 +345,7 @@ CODESTARTmodExit
 
 	/* release objects we used */
 	objRelease(glbl, CORE_COMPONENT);
+	objRelease(net, CORE_COMPONENT);
 	objRelease(datetime, CORE_COMPONENT);
 	objRelease(prop, CORE_COMPONENT);
 ENDmodExit
@@ -378,7 +363,6 @@ static rsRetVal resetConfigVariables(uchar __attribute__((unused)) *pp, void __a
 	cs.dbgPrintSymbols = 0;
 	cs.symbols_twice = 0;
 	cs.use_syscall = 0;
-	cs.symfile = NULL;
 	cs.symbol_lookup = 0;
 	cs.bPermitNonKernel = 0;
 	if(cs.pszPath != NULL) {
@@ -396,6 +380,7 @@ CODEmodInit_QueryRegCFSLineHdlr
 	CHKiRet(objUse(datetime, CORE_COMPONENT));
 	CHKiRet(objUse(glbl, CORE_COMPONENT));
 	CHKiRet(objUse(prop, CORE_COMPONENT));
+	CHKiRet(objUse(net, CORE_COMPONENT));
 
 	/* we need to create the inputName property (only once during our lifetime) */
 	CHKiRet(prop.CreateStringProp(&pInputName, UCHAR_CONSTANT("imklog"), sizeof("imklog") - 1));
@@ -405,23 +390,23 @@ CODEmodInit_QueryRegCFSLineHdlr
 	initConfigSettings();
 
 	CHKiRet(omsdRegCFSLineHdlr((uchar *)"debugprintkernelsymbols", 0, eCmdHdlrBinary,
-			NULL, &cs.dbgPrintSymbols, STD_LOADABLE_MODULE_ID, eConfObjGlobal));
+			NULL, &cs.dbgPrintSymbols, STD_LOADABLE_MODULE_ID));
 	CHKiRet(omsdRegCFSLineHdlr((uchar *)"klogpath", 0, eCmdHdlrGetWord,
-			NULL, &cs.pszPath, STD_LOADABLE_MODULE_ID, eConfObjGlobal));
+			NULL, &cs.pszPath, STD_LOADABLE_MODULE_ID));
 	CHKiRet(omsdRegCFSLineHdlr((uchar *)"klogsymbollookup", 0, eCmdHdlrBinary,
-			NULL, &cs.symbol_lookup, STD_LOADABLE_MODULE_ID, eConfObjGlobal));
+			NULL, &cs.symbol_lookup, STD_LOADABLE_MODULE_ID));
 	CHKiRet(omsdRegCFSLineHdlr((uchar *)"klogsymbolstwice", 0, eCmdHdlrBinary,
-			NULL, &cs.symbols_twice, STD_LOADABLE_MODULE_ID, eConfObjGlobal));
+			NULL, &cs.symbols_twice, STD_LOADABLE_MODULE_ID));
 	CHKiRet(omsdRegCFSLineHdlr((uchar *)"klogusesyscallinterface", 0, eCmdHdlrBinary,
-			NULL, &cs.use_syscall, STD_LOADABLE_MODULE_ID, eConfObjGlobal));
+			NULL, &cs.use_syscall, STD_LOADABLE_MODULE_ID));
 	CHKiRet(omsdRegCFSLineHdlr((uchar *)"klogpermitnonkernelfacility", 0, eCmdHdlrBinary,
-			NULL, &cs.bPermitNonKernel, STD_LOADABLE_MODULE_ID, eConfObjGlobal));
+			NULL, &cs.bPermitNonKernel, STD_LOADABLE_MODULE_ID));
 	CHKiRet(omsdRegCFSLineHdlr((uchar *)"klogconsoleloglevel", 0, eCmdHdlrInt,
-			NULL, &cs.console_log_level, STD_LOADABLE_MODULE_ID, eConfObjGlobal));
+			NULL, &cs.console_log_level, STD_LOADABLE_MODULE_ID));
 	CHKiRet(omsdRegCFSLineHdlr((uchar *)"kloginternalmsgfacility", 0, eCmdHdlrFacility,
-			NULL, &cs.iFacilIntMsg, STD_LOADABLE_MODULE_ID, eConfObjGlobal));
+			NULL, &cs.iFacilIntMsg, STD_LOADABLE_MODULE_ID));
 	CHKiRet(omsdRegCFSLineHdlr((uchar *)"resetconfigvariables", 1, eCmdHdlrCustomHandler,
-			resetConfigVariables, NULL, STD_LOADABLE_MODULE_ID, eConfObjGlobal));
+			resetConfigVariables, NULL, STD_LOADABLE_MODULE_ID));
 ENDmodInit
 /* vim:set ai:
  */
