@@ -29,7 +29,9 @@
 #include <string.h>
 #include <gnutls/gnutls.h>
 #include <gnutls/x509.h>
-#include <gcrypt.h>
+#if GNUTLS_VERSION_NUMBER <= 0x020b00
+#	include <gcrypt.h>
+#endif
 #include <errno.h>
 #include <sys/stat.h>
 #include <unistd.h>
@@ -53,7 +55,9 @@
 #define CRLFILE "crl.pem"
 
 
+#if GNUTLS_VERSION_NUMBER <= 0x020b00
 GCRY_THREAD_OPTION_PTHREAD_IMPL;
+#endif
 MODULE_TYPE_LIB
 MODULE_TYPE_KEEP
 
@@ -113,7 +117,7 @@ readFile(uchar *pszFile, gnutls_datum_t *pBuf)
 
 	pBuf->data = NULL;
 
-	if((fd = open((char*)pszFile, 0)) == -1) {
+	if((fd = open((char*)pszFile, O_RDONLY)) == -1) {
 		errmsg.LogError(0, RS_RET_FILE_NOT_FOUND, "can not read file '%s'", pszFile);
 		ABORT_FINALIZE(RS_RET_FILE_NOT_FOUND);
 
@@ -202,10 +206,14 @@ finalize_it:
 	if(iRet != RS_RET_OK) {
 		if(data.data != NULL)
 			free(data.data);
-		if(pThis->bOurCertIsInit)
+		if(pThis->bOurCertIsInit) {
 			gnutls_x509_crt_deinit(pThis->ourCert);
-		if(pThis->bOurKeyIsInit)
+			pThis->bOurCertIsInit = 0;
+		}
+		if(pThis->bOurKeyIsInit) {
 			gnutls_x509_privkey_deinit(pThis->ourKey);
+			pThis->bOurKeyIsInit = 0;
+		}
 	}
 	RETiRet;
 }
@@ -561,7 +569,9 @@ gtlsGlblInit(void)
 	DEFiRet;
 
 	/* gcry_control must be called first, so that the thread system is correctly set up */
+	#if GNUTLS_VERSION_NUMBER <= 0x020b00
 	gcry_control (GCRYCTL_SET_THREAD_CBS, &gcry_threads_pthread);
+	#endif
 	CHKgnutls(gnutls_global_init());
 	
 	/* X509 stuff */
