@@ -2,27 +2,24 @@
  *
  * Module begun 2011-04-19 by Rainer Gerhards
  *
- * Copyright 2011 by Rainer Gerhards and Adiscon GmbH.
+ * Copyright 2011-2012 Adiscon GmbH.
  *
  * This file is part of the rsyslog runtime library.
  *
- * The rsyslog runtime library is free software: you can redistribute it and/or modify
- * it under the terms of the GNU Lesser General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
- *
- * The rsyslog runtime library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public License
- * along with the rsyslog runtime library.  If not, see <http://www.gnu.org/licenses/>.
- *
- * A copy of the GPL can be found in the file "COPYING" in this distribution.
- * A copy of the LGPL can be found in the file "COPYING.LESSER" in this distribution.
+ * Licensed under the Apache License, Version 2.0 (the "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
+ * 
+ *       http://www.apache.org/licenses/LICENSE-2.0
+ *       -or-
+ *       see COPYING.ASL20 in the source distribution
+ * 
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
  */
-
 #include "config.h"
 #include <stdio.h>
 #include <stdlib.h>
@@ -96,6 +93,7 @@ static uchar template_StdUsrMsgFmt[] = "\" %syslogtag%%msg%\n\r\"";
 static uchar template_StdDBFmt[] = "\"insert into SystemEvents (Message, Facility, FromHost, Priority, DeviceReportedTime, ReceivedAt, InfoUnitID, SysLogTag) values ('%msg%', %syslogfacility%, '%HOSTNAME%', %syslogpriority%, '%timereported:::date-mysql%', '%timegenerated:::date-mysql%', %iut%, '%syslogtag%')\",SQL";
 static uchar template_StdPgSQLFmt[] = "\"insert into SystemEvents (Message, Facility, FromHost, Priority, DeviceReportedTime, ReceivedAt, InfoUnitID, SysLogTag) values ('%msg%', %syslogfacility%, '%HOSTNAME%', %syslogpriority%, '%timereported:::date-pgsql%', '%timegenerated:::date-pgsql%', %iut%, '%syslogtag%')\",STDSQL";
 static uchar template_spoofadr[] = "\"%fromhost-ip%\"";
+static uchar template_StdJSONFmt[] = "\"{\\\"message\\\":\\\"%msg:::json%\\\",\\\"fromhost\\\":\\\"%HOSTNAME:::json%\\\",\\\"facility\\\":\\\"%syslogfacility-text%\\\",\\\"priority\\\":\\\"%syslogpriority-text%\\\",\\\"timereported\\\":\\\"%timereported:::date-rfc3339%\\\",\\\"timegenerated\\\":\\\"%timegenerated:::date-rfc3339%\\\"}\"";
 /* end templates */
 
 void cnfDoCfsysline(char *ln);
@@ -240,7 +238,7 @@ cnfDoActlst(struct cnfactlst *actlst, rule_t *pRule)
 		} else {
 			dbgprintf("legacy action line:%s\n", actlst->data.legActLine);
 			str = (uchar*) actlst->data.legActLine;
-			iRet = cflineDoAction(loadConf, &str, &pAction);
+			CHKiRet(cflineDoAction(loadConf, &str, &pAction));
 			iRet = llAppend(&(pRule)->llActList,  NULL, (void*) pAction);
 		}
 		for(  cflst = actlst->syslines
@@ -249,6 +247,7 @@ cnfDoActlst(struct cnfactlst *actlst, rule_t *pRule)
 		}
 		actlst = actlst->next;
 	}
+finalize_it:
 	RETiRet;
 }
 
@@ -433,6 +432,7 @@ void cnfDoCfsysline(char *ln)
 	DBGPRINTF("cnf:global:cfsysline: %s\n", ln);
 	/* the legacy system needs the "$" stripped */
 	conf.cfsysline((uchar*) ln+1);
+	free(ln);
 }
 
 void cnfDoBSDTag(char *ln)
@@ -689,7 +689,8 @@ runInputModules(void)
 			DBGPRINTF("running module %s with config %p\n", node->pMod->pszName, node);
 			bNeedsCancel = (node->pMod->isCompatibleWithFeature(sFEATURENonCancelInputTermination) == RS_RET_OK) ?
 				       0 : 1;
-			thrdCreate(node->pMod->mod.im.runInput, node->pMod->mod.im.afterRun, bNeedsCancel);
+			thrdCreate(node->pMod->mod.im.runInput, node->pMod->mod.im.afterRun, bNeedsCancel,
+			           (node->pMod->cnfName == NULL) ? node->pMod->pszName : node->pMod->cnfName);
 		}
 		node = module.GetNxtCnfType(runConf, node, eMOD_IN);
 	}
@@ -1053,52 +1054,52 @@ initLegacyConf(void)
 
 	/* now register config handlers */
 	CHKiRet(regCfSysLineHdlr((uchar *)"sleep", 0, eCmdHdlrGoneAway,
-		NULL, NULL, NULL, eConfObjGlobal));
+		NULL, NULL, NULL));
 	CHKiRet(regCfSysLineHdlr((uchar *)"logrsyslogstatusmessages", 0, eCmdHdlrBinary,
-		NULL, &loadConf->globals.bLogStatusMsgs, NULL, eConfObjGlobal));
+		NULL, &loadConf->globals.bLogStatusMsgs, NULL));
 	CHKiRet(regCfSysLineHdlr((uchar *)"errormessagestostderr", 0, eCmdHdlrBinary,
-		NULL, &loadConf->globals.bErrMsgToStderr, NULL, eConfObjGlobal));
+		NULL, &loadConf->globals.bErrMsgToStderr, NULL));
 	CHKiRet(regCfSysLineHdlr((uchar *)"abortonuncleanconfig", 0, eCmdHdlrBinary,
-		NULL, &loadConf->globals.bAbortOnUncleanConfig, NULL, eConfObjGlobal));
+		NULL, &loadConf->globals.bAbortOnUncleanConfig, NULL));
 	CHKiRet(regCfSysLineHdlr((uchar *)"repeatedmsgreduction", 0, eCmdHdlrBinary,
-		NULL, &loadConf->globals.bReduceRepeatMsgs, NULL, eConfObjGlobal));
+		NULL, &loadConf->globals.bReduceRepeatMsgs, NULL));
 	CHKiRet(regCfSysLineHdlr((uchar *)"debugprinttemplatelist", 0, eCmdHdlrBinary,
-		NULL, &(loadConf->globals.bDebugPrintTemplateList), NULL, eConfObjGlobal));
+		NULL, &(loadConf->globals.bDebugPrintTemplateList), NULL));
 	CHKiRet(regCfSysLineHdlr((uchar *)"debugprintmodulelist", 0, eCmdHdlrBinary,
-		NULL, &(loadConf->globals.bDebugPrintModuleList), NULL, eConfObjGlobal));
+		NULL, &(loadConf->globals.bDebugPrintModuleList), NULL));
 	CHKiRet(regCfSysLineHdlr((uchar *)"debugprintcfsyslinehandlerlist", 0, eCmdHdlrBinary,
-		 NULL, &(loadConf->globals.bDebugPrintCfSysLineHandlerList), NULL, eConfObjGlobal));
+		 NULL, &(loadConf->globals.bDebugPrintCfSysLineHandlerList), NULL));
 	CHKiRet(regCfSysLineHdlr((uchar *)"privdroptouser", 0, eCmdHdlrUID,
-		NULL, &loadConf->globals.uidDropPriv, NULL, eConfObjGlobal));
+		NULL, &loadConf->globals.uidDropPriv, NULL));
 	CHKiRet(regCfSysLineHdlr((uchar *)"privdroptouserid", 0, eCmdHdlrInt,
-		NULL, &loadConf->globals.uidDropPriv, NULL, eConfObjGlobal));
+		NULL, &loadConf->globals.uidDropPriv, NULL));
 	CHKiRet(regCfSysLineHdlr((uchar *)"privdroptogroup", 0, eCmdHdlrGID,
-		NULL, &loadConf->globals.gidDropPriv, NULL, eConfObjGlobal));
+		NULL, &loadConf->globals.gidDropPriv, NULL));
 	CHKiRet(regCfSysLineHdlr((uchar *)"privdroptogroupid", 0, eCmdHdlrGID,
-		NULL, &loadConf->globals.gidDropPriv, NULL, eConfObjGlobal));
+		NULL, &loadConf->globals.gidDropPriv, NULL));
 	CHKiRet(regCfSysLineHdlr((uchar *)"generateconfiggraph", 0, eCmdHdlrGetWord,
-		NULL, &loadConf->globals.pszConfDAGFile, NULL, eConfObjGlobal));
+		NULL, &loadConf->globals.pszConfDAGFile, NULL));
 	CHKiRet(regCfSysLineHdlr((uchar *)"umask", 0, eCmdHdlrFileCreateMode,
-		NULL, &loadConf->globals.umask, NULL, eConfObjGlobal));
+		NULL, &loadConf->globals.umask, NULL));
 	CHKiRet(regCfSysLineHdlr((uchar *)"maxopenfiles", 0, eCmdHdlrInt,
-		setMaxFiles, NULL, NULL, eConfObjGlobal));
+		setMaxFiles, NULL, NULL));
 
 	CHKiRet(regCfSysLineHdlr((uchar *)"actionresumeinterval", 0, eCmdHdlrInt,
-		setActionResumeInterval, NULL, NULL, eConfObjGlobal));
+		setActionResumeInterval, NULL, NULL));
 	CHKiRet(regCfSysLineHdlr((uchar *)"modload", 0, eCmdHdlrCustomHandler,
-		conf.doModLoad, NULL, NULL, eConfObjGlobal));
+		conf.doModLoad, NULL, NULL));
 	CHKiRet(regCfSysLineHdlr((uchar *)"defaultruleset", 0, eCmdHdlrGetWord,
-		setDefaultRuleset, NULL, NULL, eConfObjGlobal));
+		setDefaultRuleset, NULL, NULL));
 	CHKiRet(regCfSysLineHdlr((uchar *)"ruleset", 0, eCmdHdlrGetWord,
-		setCurrRuleset, NULL, NULL, eConfObjGlobal));
+		setCurrRuleset, NULL, NULL));
 
 	/* handler for "larger" config statements (tie into legacy conf system) */
 	CHKiRet(regCfSysLineHdlr((uchar *)"template", 0, eCmdHdlrCustomHandler,
-		conf.doNameLine, (void*)DIR_TEMPLATE, NULL, eConfObjGlobal));
+		conf.doNameLine, (void*)DIR_TEMPLATE, NULL));
 	CHKiRet(regCfSysLineHdlr((uchar *)"outchannel", 0, eCmdHdlrCustomHandler,
-		conf.doNameLine, (void*)DIR_OUTCHANNEL, NULL, eConfObjGlobal));
+		conf.doNameLine, (void*)DIR_OUTCHANNEL, NULL));
 	CHKiRet(regCfSysLineHdlr((uchar *)"allowedsender", 0, eCmdHdlrCustomHandler,
-		conf.doNameLine, (void*)DIR_ALLOWEDSENDER, NULL, eConfObjGlobal));
+		conf.doNameLine, (void*)DIR_ALLOWEDSENDER, NULL));
 
 	/* the following are parameters for the main message queue. I have the
 	 * strong feeling that this needs to go to a different space, but that
@@ -1106,49 +1107,49 @@ initLegacyConf(void)
 	 * rgerhards, 2011-04-21
 	 */
 	CHKiRet(regCfSysLineHdlr((uchar *)"mainmsgqueuefilename", 0, eCmdHdlrGetWord,
-		NULL, &loadConf->globals.mainQ.pszMainMsgQFName, NULL, eConfObjGlobal));
+		NULL, &loadConf->globals.mainQ.pszMainMsgQFName, NULL));
 	CHKiRet(regCfSysLineHdlr((uchar *)"mainmsgqueuesize", 0, eCmdHdlrInt,
-		NULL, &loadConf->globals.mainQ.iMainMsgQueueSize, NULL, eConfObjGlobal));
+		NULL, &loadConf->globals.mainQ.iMainMsgQueueSize, NULL));
 	CHKiRet(regCfSysLineHdlr((uchar *)"mainmsgqueuehighwatermark", 0, eCmdHdlrInt,
-		NULL, &loadConf->globals.mainQ.iMainMsgQHighWtrMark, NULL, eConfObjGlobal));
+		NULL, &loadConf->globals.mainQ.iMainMsgQHighWtrMark, NULL));
 	CHKiRet(regCfSysLineHdlr((uchar *)"mainmsgqueuelowwatermark", 0, eCmdHdlrInt,
-		NULL, &loadConf->globals.mainQ.iMainMsgQLowWtrMark, NULL, eConfObjGlobal));
+		NULL, &loadConf->globals.mainQ.iMainMsgQLowWtrMark, NULL));
 	CHKiRet(regCfSysLineHdlr((uchar *)"mainmsgqueuediscardmark", 0, eCmdHdlrInt,
-		NULL, &loadConf->globals.mainQ.iMainMsgQDiscardMark, NULL, eConfObjGlobal));
+		NULL, &loadConf->globals.mainQ.iMainMsgQDiscardMark, NULL));
 	CHKiRet(regCfSysLineHdlr((uchar *)"mainmsgqueuediscardseverity", 0, eCmdHdlrSeverity,
-		NULL, &loadConf->globals.mainQ.iMainMsgQDiscardSeverity, NULL, eConfObjGlobal));
+		NULL, &loadConf->globals.mainQ.iMainMsgQDiscardSeverity, NULL));
 	CHKiRet(regCfSysLineHdlr((uchar *)"mainmsgqueuecheckpointinterval", 0, eCmdHdlrInt,
-		NULL, &loadConf->globals.mainQ.iMainMsgQPersistUpdCnt, NULL, eConfObjGlobal));
+		NULL, &loadConf->globals.mainQ.iMainMsgQPersistUpdCnt, NULL));
 	CHKiRet(regCfSysLineHdlr((uchar *)"mainmsgqueuesyncqueuefiles", 0, eCmdHdlrBinary,
-		NULL, &loadConf->globals.mainQ.bMainMsgQSyncQeueFiles, NULL, eConfObjGlobal));
+		NULL, &loadConf->globals.mainQ.bMainMsgQSyncQeueFiles, NULL));
 	CHKiRet(regCfSysLineHdlr((uchar *)"mainmsgqueuetype", 0, eCmdHdlrGetWord,
-		setMainMsgQueType, NULL, NULL, eConfObjGlobal));
+		setMainMsgQueType, NULL, NULL));
 	CHKiRet(regCfSysLineHdlr((uchar *)"mainmsgqueueworkerthreads", 0, eCmdHdlrInt,
-		NULL, &loadConf->globals.mainQ.iMainMsgQueueNumWorkers, NULL, eConfObjGlobal));
+		NULL, &loadConf->globals.mainQ.iMainMsgQueueNumWorkers, NULL));
 	CHKiRet(regCfSysLineHdlr((uchar *)"mainmsgqueuetimeoutshutdown", 0, eCmdHdlrInt,
-		NULL, &loadConf->globals.mainQ.iMainMsgQtoQShutdown, NULL, eConfObjGlobal));
+		NULL, &loadConf->globals.mainQ.iMainMsgQtoQShutdown, NULL));
 	CHKiRet(regCfSysLineHdlr((uchar *)"mainmsgqueuetimeoutactioncompletion", 0, eCmdHdlrInt,
-		NULL, &loadConf->globals.mainQ.iMainMsgQtoActShutdown, NULL, eConfObjGlobal));
+		NULL, &loadConf->globals.mainQ.iMainMsgQtoActShutdown, NULL));
 	CHKiRet(regCfSysLineHdlr((uchar *)"mainmsgqueuetimeoutenqueue", 0, eCmdHdlrInt,
-		NULL, &loadConf->globals.mainQ.iMainMsgQtoEnq, NULL, eConfObjGlobal));
+		NULL, &loadConf->globals.mainQ.iMainMsgQtoEnq, NULL));
 	CHKiRet(regCfSysLineHdlr((uchar *)"mainmsgqueueworkertimeoutthreadshutdown", 0, eCmdHdlrInt,
-		NULL, &loadConf->globals.mainQ.iMainMsgQtoWrkShutdown, NULL, eConfObjGlobal));
+		NULL, &loadConf->globals.mainQ.iMainMsgQtoWrkShutdown, NULL));
 	CHKiRet(regCfSysLineHdlr((uchar *)"mainmsgqueuedequeueslowdown", 0, eCmdHdlrInt,
-		NULL, &loadConf->globals.mainQ.iMainMsgQDeqSlowdown, NULL, eConfObjGlobal));
+		NULL, &loadConf->globals.mainQ.iMainMsgQDeqSlowdown, NULL));
 	CHKiRet(regCfSysLineHdlr((uchar *)"mainmsgqueueworkerthreadminimummessages", 0, eCmdHdlrInt,
-		NULL, &loadConf->globals.mainQ.iMainMsgQWrkMinMsgs, NULL, eConfObjGlobal));
+		NULL, &loadConf->globals.mainQ.iMainMsgQWrkMinMsgs, NULL));
 	CHKiRet(regCfSysLineHdlr((uchar *)"mainmsgqueuemaxfilesize", 0, eCmdHdlrSize,
-		NULL, &loadConf->globals.mainQ.iMainMsgQueMaxFileSize, NULL, eConfObjGlobal));
+		NULL, &loadConf->globals.mainQ.iMainMsgQueMaxFileSize, NULL));
 	CHKiRet(regCfSysLineHdlr((uchar *)"mainmsgqueuedequeuebatchsize", 0, eCmdHdlrSize,
-		NULL, &loadConf->globals.mainQ.iMainMsgQueDeqBatchSize, NULL, eConfObjGlobal));
+		NULL, &loadConf->globals.mainQ.iMainMsgQueDeqBatchSize, NULL));
 	CHKiRet(regCfSysLineHdlr((uchar *)"mainmsgqueuemaxdiskspace", 0, eCmdHdlrSize,
-		NULL, &loadConf->globals.mainQ.iMainMsgQueMaxDiskSpace, NULL, eConfObjGlobal));
+		NULL, &loadConf->globals.mainQ.iMainMsgQueMaxDiskSpace, NULL));
 	CHKiRet(regCfSysLineHdlr((uchar *)"mainmsgqueuesaveonshutdown", 0, eCmdHdlrBinary,
-		NULL, &loadConf->globals.mainQ.bMainMsgQSaveOnShutdown, NULL, eConfObjGlobal));
+		NULL, &loadConf->globals.mainQ.bMainMsgQSaveOnShutdown, NULL));
 	CHKiRet(regCfSysLineHdlr((uchar *)"mainmsgqueuedequeuetimebegin", 0, eCmdHdlrInt,
-		NULL, &loadConf->globals.mainQ.iMainMsgQueueDeqtWinFromHr, NULL, eConfObjGlobal));
+		NULL, &loadConf->globals.mainQ.iMainMsgQueueDeqtWinFromHr, NULL));
 	CHKiRet(regCfSysLineHdlr((uchar *)"mainmsgqueuedequeuetimeend", 0, eCmdHdlrInt,
-		NULL, &loadConf->globals.mainQ.iMainMsgQueueDeqtWinToHr, NULL, eConfObjGlobal));
+		NULL, &loadConf->globals.mainQ.iMainMsgQueueDeqtWinToHr, NULL));
 	/* moddir is a bit hard problem -- because it actually needs to
 	 * modify a setting that is specific to module.c. The important point
 	 * is that this action MUST actually be carried out during config load,
@@ -1157,11 +1158,11 @@ initLegacyConf(void)
 	 * TODO: think about a clean solution
 	 */
 	CHKiRet(regCfSysLineHdlr((uchar *)"moddir", 0, eCmdHdlrGetWord,
-		setModDir, NULL, NULL, eConfObjGlobal));
+		setModDir, NULL, NULL));
 
 	/* finally, the reset handler */
 	CHKiRet(regCfSysLineHdlr((uchar *)"resetconfigvariables", 1, eCmdHdlrCustomHandler,
-		resetConfigVariables, NULL, NULL, eConfObjGlobal));
+		resetConfigVariables, NULL, NULL));
 
 	/* initialize the build-in templates */
 	pTmp = template_DebugFormat;
@@ -1184,6 +1185,8 @@ initLegacyConf(void)
 	tplAddLine(ourConf, " StdDBFmt", &pTmp);
         pTmp = template_StdPgSQLFmt;
         tplAddLine(ourConf, " StdPgSQLFmt", &pTmp);
+        pTmp = template_StdJSONFmt;
+        tplAddLine(ourConf, " StdJSONFmt", &pTmp);
         pTmp = template_spoofadr;
         tplLastStaticInit(ourConf, tplAddLine(ourConf, "RSYSLOG_omudpspoofDfltSourceTpl", &pTmp));
 
