@@ -60,12 +60,12 @@
 						 * rgerhards, 2006-11-30
 						 */
 
-#define CONF_OMOD_NUMSTRINGS_MAXSIZE	2	/* cache for pointers to output module buffer pointers. All
-						 * rsyslog-provided plugins do NOT need more than two buffers. If
+#define CONF_OMOD_NUMSTRINGS_MAXSIZE	3	/* cache for pointers to output module buffer pointers. All
+						 * rsyslog-provided plugins do NOT need more than three buffers. If
 						 * more are needed (future developments, third-parties), rsyslog 
 						 * must be recompiled with a larger parameter. Hardcoding this
 						 * saves us some overhead, both in runtime in code complexity. As
-						 * it is doubtful if ever more than 2 parameters are needed, the
+						 * it is doubtful if ever more than 3 parameters are needed, the
 						 * approach taken here is considered appropriate.
 						 * rgerhards, 2010-06-24
 						 */
@@ -128,6 +128,7 @@ typedef uintTiny	propid_t;
 #define PROP_APP_NAME			20
 #define PROP_PROCID			21
 #define PROP_MSGID			22
+#define PROP_PARSESUCCESS		23
 #define PROP_SYS_NOW			150
 #define PROP_SYS_YEAR			151
 #define PROP_SYS_MONTH			152
@@ -140,6 +141,7 @@ typedef uintTiny	propid_t;
 #define PROP_CEE			200
 #define PROP_CEE_ALL_JSON		201
 #define PROP_SYS_BOM			159
+#define PROP_SYS_UPTIME			160
 
 
 /* The error codes below are orginally "borrowed" from
@@ -352,6 +354,8 @@ enum rsRetVal_				/** return value. All methods return this if not specified oth
 	RS_RET_WRN_WRKDIR = -2182, /**< correctable problems with the rsyslog working directory */
 	RS_RET_ERR_QUEUE_EMERGENCY = -2183, /**<  some fatal error caused queue to switch to emergency mode */
 	RS_RET_OUTDATED_STMT = -2184, /**<  some outdated statement/functionality is being used in conf file */
+	RS_RET_MISSING_WHITESPACE = -2185, /**<  whitespace is missing in some config construct */
+	RS_RET_OK_WARN = -2186, /**<  config part: everything was OK, but a warning message was emitted */
 
 	RS_RET_INVLD_CONF_OBJ= -2200,	/**< invalid config object (e.g. $Begin conf statement) */
 	RS_RET_ERR_LIBEE_INIT = -2201,	/**< cannot obtain libee ctx */
@@ -366,6 +370,11 @@ enum rsRetVal_				/** return value. All methods return this if not specified oth
 	RS_RET_CONFOBJ_UNSUPPORTED = -2210,/**< config objects (v6 conf) are not supported here */
 	RS_RET_MISSING_CNFPARAMS = -2211, /**< missing configuration parameters */
 	RS_RET_NO_LISTNERS = -2212, /**< module loaded, but no listeners are defined */
+	RS_RET_INVLD_PROTOCOL = -2213, /**< invalid protocol specified in config file */
+	RS_RET_CNF_INVLD_FRAMING = -2214, /**< invalid framing specified in config file */
+	RS_RET_LEGA_ACT_NOT_SUPPORTED = -2215, /**< the module (no longer) supports legacy action syntax */
+	RS_RET_MAX_OMSR_REACHED = -2216, /**< max nbr of string requests reached, not supported by core */
+	RS_RET_UID_MISSING = -2217,	/**< a user id is missing (but e.g. a password provided) */
 
 	/* RainerScript error messages (range 1000.. 1999) */
 	RS_RET_SYSVAR_NOT_FOUND = 1001, /**< system variable could not be found (maybe misspelled) */
