@@ -3,25 +3,23 @@
  * This object provides a statistics-gathering facility inside rsyslog. This
  * functionality will be pragmatically implemented and extended.
  *
- * Copyright 2010 Rainer Gerhards and Adiscon GmbH.
+ * Copyright 2010-2012 Adiscon GmbH.
  *
  * This file is part of the rsyslog runtime library.
  *
- * The rsyslog runtime library is free software: you can redistribute it and/or modify
- * it under the terms of the GNU Lesser General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
- *
- * The rsyslog runtime library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public License
- * along with the rsyslog runtime library.  If not, see <http://www.gnu.org/licenses/>.
- *
- * A copy of the GPL can be found in the file "COPYING" in this distribution.
- * A copy of the LGPL can be found in the file "COPYING.LESSER" in this distribution.
+ * Licensed under the Apache License, Version 2.0 (the "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
+ * 
+ *       http://www.apache.org/licenses/LICENSE-2.0
+ *       -or-
+ *       see COPYING.ASL20 in the source distribution
+ * 
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
  */
 
 #include "config.h"
@@ -138,6 +136,10 @@ finalize_it:
 
 /* add a counter to an object
  * ctrName is duplicated, caller must free it if requried
+ * NOTE: The counter is READ-ONLY and MUST NOT be modified (most
+ * importantly, it must not be initialized, so the caller must
+ * ensure the counter is properly initialized before AddCounter()
+ * is called.
  */
 static rsRetVal
 addCounter(statsobj_t *pThis, uchar *ctrName, statsCtrType_t ctrType, void *pCtr)
@@ -164,6 +166,56 @@ finalize_it:
 	RETiRet;
 }
 
+/* get all the object's countes together as CEE. */
+static rsRetVal
+getStatsLineCEE(statsobj_t *pThis, cstr_t **ppcstr)
+{
+	cstr_t *pcstr;
+	ctr_t *pCtr;
+	DEFiRet;
+
+	CHKiRet(cstrConstruct(&pcstr));
+	rsCStrAppendStrWithLen(pcstr, UCHAR_CONSTANT("@cee: {"), 7);
+
+	rsCStrAppendStrWithLen(pcstr, UCHAR_CONSTANT("\""), 1);
+	rsCStrAppendStrWithLen(pcstr, UCHAR_CONSTANT("name"), 4);
+	rsCStrAppendStrWithLen(pcstr, UCHAR_CONSTANT("\""), 1);
+	rsCStrAppendStrWithLen(pcstr, UCHAR_CONSTANT(":"), 1);
+	rsCStrAppendStrWithLen(pcstr, UCHAR_CONSTANT("\""), 1);
+	rsCStrAppendStr(pcstr, pThis->name);
+	rsCStrAppendStrWithLen(pcstr, UCHAR_CONSTANT("\""), 1);
+	rsCStrAppendStrWithLen(pcstr, UCHAR_CONSTANT(","), 1);
+
+	/* now add all counters to this line */
+	pthread_mutex_lock(&pThis->mutCtr);
+	for(pCtr = pThis->ctrRoot ; pCtr != NULL ; pCtr = pCtr->next) {
+		rsCStrAppendStrWithLen(pcstr, UCHAR_CONSTANT("\""), 1);
+		rsCStrAppendStr(pcstr, pCtr->name);
+		rsCStrAppendStrWithLen(pcstr, UCHAR_CONSTANT("\""), 1);
+		cstrAppendChar(pcstr, ':');
+		switch(pCtr->ctrType) {
+		case ctrType_IntCtr:
+			rsCStrAppendInt(pcstr, *(pCtr->val.pIntCtr)); // TODO: OK?????
+			break;
+		case ctrType_Int:
+			rsCStrAppendInt(pcstr, *(pCtr->val.pInt));
+			break;
+		}
+		if (pCtr->next != NULL) {
+			cstrAppendChar(pcstr, ',');
+		} else {
+			cstrAppendChar(pcstr, '}');
+		}
+
+	}
+	pthread_mutex_unlock(&pThis->mutCtr);
+
+	CHKiRet(cstrFinalize(pcstr));
+	*ppcstr = pcstr;
+
+finalize_it:
+	RETiRet;
+}
 
 /* get all the object's countes together with object name as one line.
  */
@@ -210,14 +262,21 @@ finalize_it:
  * line. If the callback reports an error, processing is stopped.
  */
 static rsRetVal
-getAllStatsLines(rsRetVal(*cb)(void*, cstr_t*), void *usrptr)
+getAllStatsLines(rsRetVal(*cb)(void*, cstr_t*), void *usrptr, statsFmtType_t fmt)
 {
 	statsobj_t *o;
 	cstr_t *cstr;
 	DEFiRet;
 
 	for(o = objRoot ; o != NULL ; o = o->next) {
-		CHKiRet(getStatsLine(o, &cstr));
+		switch(fmt) {
+		case statsFmt_Legacy:
+			CHKiRet(getStatsLine(o, &cstr));
+			break;
+		case statsFmt_JSON:
+			CHKiRet(getStatsLineCEE(o, &cstr));
+			break;
+		}
 		CHKiRet(cb(usrptr, cstr));
 		rsCStrDestruct(&cstr);
 	}
