@@ -5,30 +5,26 @@
  * NOTE: read comments in module-template.h to understand how this file
  *       works!
  *
- * File begun on 2007-07-20 by RGerhards (extracted from syslogd.c)
- * This file is under development and has not yet arrived at being fully
- * self-contained and a real object. So far, it is mostly an excerpt
- * of the "old" message code without any modifications. However, it
- * helps to have things at the right place one we go to the meat of it.
+ * File begun on 2007-07-20 by RGerhards (extracted from syslogd.c, which at the
+ * time of the fork from sysklogd was under BSD license)
  *
- * Copyright 2007-2011 Rainer Gerhards and Adiscon GmbH.
+ * Copyright 2007-2012 Adiscon GmbH.
  *
  * This file is part of rsyslog.
  *
- * Rsyslog is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
- *
- * Rsyslog is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with Rsyslog.  If not, see <http://www.gnu.org/licenses/>.
- *
- * A copy of the GPL can be found in the file "COPYING" in this distribution.
+ * Licensed under the Apache License, Version 2.0 (the "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
+ * 
+ *       http://www.apache.org/licenses/LICENSE-2.0
+ *       -or-
+ *       see COPYING.ASL20 in the source distribution
+ * 
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
  */
 #include "config.h"
 #include "rsyslog.h"
@@ -94,8 +90,7 @@ typedef struct _instanceData {
 typedef struct configSettings_s {
 	EMPTY_STRUCT
 } configSettings_t;
-
-SCOPING_SUPPORT; /* must be set AFTER configSettings_t is defined */
+static configSettings_t __attribute__((unused)) cs;
 
 
 /* tables for interfacing with the v6 config system */
@@ -385,9 +380,9 @@ CODE_STD_FINALIZERnewActInst
 ENDnewActInst
 
 
-
 BEGINparseSelectorAct
 	es_str_t *usrs;
+	int bHadWarning;
 CODESTARTparseSelectorAct
 CODE_STD_STRING_REQUESTparseSelectorAct(1)
 	if(!strncmp((char*) p, ":omusrmsg:", sizeof(":omusrmsg:") - 1)) {
@@ -395,12 +390,13 @@ CODE_STD_STRING_REQUESTparseSelectorAct(1)
 	} else {
 		if(!*p || !((*p >= 'a' && *p <= 'z') || (*p >= 'A' && *p <= 'Z')
 	   || (*p >= '0' && *p <= '9') || *p == '_' || *p == '.' || *p == '*')) {
-		ABORT_FINALIZE(RS_RET_CONFLINE_UNPROCESSED);
+			ABORT_FINALIZE(RS_RET_CONFLINE_UNPROCESSED);
 		} else {
 			errmsg.LogError(0, RS_RET_OUTDATED_STMT,
 			   "action '%s' treated as ':omusrmsg:%s' - please "
 			   "change syntax, '%s' will not be supported in the future",
 			   p, p, p);
+			bHadWarning = 1;
 		}
 	}
 
@@ -424,6 +420,8 @@ CODE_STD_STRING_REQUESTparseSelectorAct(1)
 			!= RS_RET_OK)
 			goto finalize_it;
 	}
+	if(iRet == RS_RET_OK && bHadWarning)
+		iRet = RS_RET_OK_WARN;
 CODE_STD_FINALIZERparseSelectorAct
 ENDparseSelectorAct
 
@@ -442,7 +440,7 @@ ENDqueryEtryPt
 
 BEGINmodInit(UsrMsg)
 CODESTARTmodInit
-SCOPINGmodInit
+INITLegCnfVars
 	*ipIFVersProvided = CURR_MOD_IF_VERSION; /* we only support the current interface specification */
 CODEmodInit_QueryRegCFSLineHdlr
 	CHKiRet(objUse(errmsg, CORE_COMPONENT));
