@@ -21,7 +21,7 @@
  * For further information, please see http://www.rsyslog.com
  *
  * rsyslog - An Enhanced syslogd Replacement.
- * Copyright 2003-2011 Rainer Gerhards and Adiscon GmbH.
+ * Copyright 2003-2012 Rainer Gerhards and Adiscon GmbH.
  *
  * This file is part of rsyslog.
  *
@@ -143,6 +143,7 @@ DEFobjCurrIf(net) /* TODO: make go away! */
 
 /* forward definitions */
 static rsRetVal GlobalClassExit(void);
+static rsRetVal queryLocalHostname(void);
 
 
 #ifndef _PATH_LOGCONF 
@@ -190,7 +191,6 @@ static rsRetVal GlobalClassExit(void);
 rsconf_t *ourConf;				/* our config object */
 
 static prop_t *pInternalInputName = NULL;	/* there is only one global inputName for all internally-generated messages */
-static prop_t *pLocalHostIP = NULL;		/* there is only one global IP for all internally-generated messages */
 static uchar	*ConfFile = (uchar*) _PATH_LOGCONF; /* read-only after startup */
 static char	*PidFile = _PATH_LOGPID; /* read-only after startup */
 
@@ -356,8 +356,15 @@ void untty(void)
 #else
 {
 	int i;
+	pid_t pid;
 
 	if(!Debug) {
+		pid = getpid();
+		if (setpgid(pid, pid) < 0) {
+			perror("setpgid");
+			exit(1);
+		}
+
 		i = open(_PATH_TTY, O_RDWR|O_CLOEXEC);
 		if (i >= 0) {
 #			if !defined(__hpux)
@@ -441,7 +448,8 @@ logmsgInternal(int iErr, int pri, uchar *msg, int flags)
 	MsgSetRawMsgWOSize(pMsg, (char*)msg);
 	MsgSetHOSTNAME(pMsg, glbl.GetLocalHostName(), ustrlen(glbl.GetLocalHostName()));
 	MsgSetRcvFrom(pMsg, glbl.GetLocalHostNameProp());
-	MsgSetRcvFromIP(pMsg, pLocalHostIP);
+dbgprintf("ZZZZ: pLocalHostIPIF used!\n");
+	MsgSetRcvFromIP(pMsg, glbl.GetLocalHostIP());
 	MsgSetMSGoffs(pMsg, 0);
 	/* check if we have an error code associated and, if so,
 	 * adjust the tag. -- rgerhards, 2008-06-27
@@ -622,11 +630,19 @@ submitMsg(msg_t *pMsg)
 	ISOBJ_TYPE_assert(pMsg, msg);
 
 	pRuleset = MsgGetRuleset(pMsg);
-
 	pQueue = (pRuleset == NULL) ? pMsgQueue : ruleset.GetRulesetQueue(pRuleset);
+
+	/* if a plugin logs a message during shutdown, the queue may no longer exist */
+	if(pQueue == NULL) {
+		DBGPRINTF("submitMsg() could not submit message - "
+			  "queue does (no longer?) exist - ignored\n");
+		FINALIZE;
+	}
+
 	MsgPrepareEnqueue(pMsg);
 	qqueueEnqObj(pQueue, pMsg->flowCtlType, (void*) pMsg);
 
+finalize_it:
 	RETiRet;
 }
 
@@ -647,12 +663,20 @@ multiSubmitMsg(multi_submit_t *pMultiSub)
 	if(pMultiSub->nElem == 0)
 		FINALIZE;
 
+	pRuleset = MsgGetRuleset(pMultiSub->ppMsgs[0]);
+	pQueue = (pRuleset == NULL) ? pMsgQueue : ruleset.GetRulesetQueue(pRuleset);
+
+	/* if a plugin logs a message during shutdown, the queue may no longer exist */
+	if(pQueue == NULL) {
+		DBGPRINTF("multiSubmitMsg() could not submit message - "
+			  "queue does (no longer?) exist - ignored\n");
+		FINALIZE;
+	}
+
 	for(i = 0 ; i < pMultiSub->nElem ; ++i) {
 		MsgPrepareEnqueue(pMultiSub->ppMsgs[i]);
 	}
 
-	pRuleset = MsgGetRuleset(pMultiSub->ppMsgs[0]);
-	pQueue = (pRuleset == NULL) ? pMsgQueue : ruleset.GetRulesetQueue(pRuleset);
 	iRet = pQueue->MultiEnq(pQueue, pMultiSub);
 	pMultiSub->nElem = 0;
 
@@ -688,7 +712,7 @@ DEFFUNC_llExecFunc(flushRptdMsgsActions)
 	assert(pAction != NULL);
 
 	BEGINfunc
-	LockObj(pAction);
+	d_pthread_mutex_lock(&pAction->mutAction);
 	/* TODO: time() performance: the call below could be moved to
 	 * the beginn of the llExec(). This makes it slightly less correct, but
 	 * in an acceptable way. -- rgerhards, 2008-09-16
@@ -700,7 +724,7 @@ DEFFUNC_llExecFunc(flushRptdMsgsActions)
 		actionWriteToAction(pAction);
 		BACKOFF(pAction);
 	}
-	UnlockObj(pAction);
+	d_pthread_mutex_unlock(&pAction->mutAction);
 
 	ENDfunc
 	return RS_RET_OK; /* we ignore errors, we can not do anything either way */
@@ -841,6 +865,11 @@ die(int sig)
 		errno = 0;
 		logmsgInternal(NO_ERRCODE, LOG_SYSLOG|LOG_INFO, (uchar*)buf, 0);
 	}
+	/* we sleep for 50ms to give the queue a chance to pick up the exit message;
+	 * otherwise we have seen cases where the message did not make it to log
+	 * files, even on idle systems.
+	 */
+	srSleep(0, 50);
 
 	/* drain queue (if configured so) and stop main queue worker thread pool */
 	DBGPRINTF("Terminating main queue...\n");
@@ -875,8 +904,6 @@ die(int sig)
 	/* destruct our global properties */
 	if(pInternalInputName != NULL)
 		prop.Destruct(&pInternalInputName);
-	if(pLocalHostIP != NULL)
-		prop.Destruct(&pLocalHostIP);
 
 	/* terminate the remaining classes */
 	GlobalClassExit();
@@ -1251,6 +1278,11 @@ DEFFUNC_llExecFunc(doHUPActions)
  * is *NOT* the sighup handler. The signal is recorded by the handler, that record
  * detected inside the mainloop and then this function is called to do the
  * real work. -- rgerhards, 2008-10-22
+ * Note: there is a VERY slim chance of a data race when the hostname is reset.
+ * We prefer to take this risk rather than sync all accesses, because to the best
+ * of my analysis it can not really hurt (the actual property is reference-counted)
+ * but the sync would require some extra CPU for *each* message processed.
+ * rgerhards, 2012-04-11
  */
 static inline void
 doHUP(void)
@@ -1266,6 +1298,7 @@ doHUP(void)
 		logmsgInternal(NO_ERRCODE, LOG_SYSLOG|LOG_INFO, (uchar*)buf, 0);
 	}
 
+	queryLocalHostname(); /* re-read our name */
 	ruleset.IterateAllActions(ourConf, doHUPActions, NULL);
 }
 
@@ -1368,7 +1401,7 @@ static void printVersion(void)
 #else
 	printf("\t32bit Atomic operations supported:\tNo\n");
 #endif
-#ifdef	HAVE_ATOMIC_BUILTINS64
+#ifdef	HAVE_ATOMIC_BUILTINS_64BIT
 	printf("\t64bit Atomic operations supported:\tYes\n");
 #else
 	printf("\t64bit Atomic operations supported:\tNo\n");
@@ -1382,7 +1415,6 @@ static void printVersion(void)
 }
 
 
-
 /* Method to initialize all global classes and use the objects that we need.
  * rgerhards, 2008-01-04
  * rgerhards, 2008-04-16: the actual initialization is now carried out by the runtime
@@ -1483,6 +1515,93 @@ GlobalClassExit(void)
 }
 
 
+/* query our host and domain names - we need to do this early as we may emit
+ * rgerhards, 2012-04-11
+ */
+static rsRetVal
+queryLocalHostname(void)
+{
+	uchar *LocalHostName;
+	uchar *LocalDomain;
+	uchar *LocalFQDNName;
+	uchar *p;
+	struct hostent *hent;
+	DEFiRet;
+
+	net.getLocalHostname(&LocalFQDNName);
+	CHKmalloc(LocalHostName = (uchar*) strdup((char*)LocalFQDNName));
+	glbl.SetLocalFQDNName(LocalFQDNName); /* set the FQDN before we modify it */
+	if((p = (uchar*)strchr((char*)LocalHostName, '.'))) {
+		*p++ = '\0';
+		LocalDomain = p;
+	} else {
+		LocalDomain = (uchar*)"";
+
+		/* It's not clearly defined whether gethostname()
+		 * should return the simple hostname or the fqdn. A
+		 * good piece of software should be aware of both and
+		 * we want to distribute good software.  Joey
+		 *
+		 * Good software also always checks its return values...
+		 * If syslogd starts up before DNS is up & /etc/hosts
+		 * doesn't have LocalHostName listed, gethostbyname will
+                * return NULL.
+		 */
+		/* TODO: gethostbyname() is not thread-safe, but replacing it is
+		 * not urgent as we do not run on multiple threads here. rgerhards, 2007-09-25
+		 */
+		hent = gethostbyname((char*)LocalHostName);
+		if(hent) {
+			int i = 0;
+
+			if(hent->h_aliases) {
+				size_t hnlen;
+
+				hnlen = strlen((char *) LocalHostName);
+
+				for (i = 0; hent->h_aliases[i]; i++) {
+					if (!strncmp(hent->h_aliases[i], (char *) LocalHostName, hnlen)
+					    && hent->h_aliases[i][hnlen] == '.') {
+						/* found a matching hostname */
+						break;
+					}
+				}
+			}
+
+			free(LocalHostName);
+			if(hent->h_aliases && hent->h_aliases[i]) {
+				CHKmalloc(LocalHostName = (uchar*)strdup(hent->h_aliases[i]));
+			} else {
+				CHKmalloc(LocalHostName = (uchar*)strdup(hent->h_name));
+			}
+
+			if((p = (uchar*)strchr((char*)LocalHostName, '.')))
+			{
+				*p++ = '\0';
+				LocalDomain = p;
+			}
+		}
+	}
+
+	/* LocalDomain is "" or part of LocalHostName, allocate a new string */
+	CHKmalloc(LocalDomain = (uchar*)strdup((char*)LocalDomain));
+
+	/* Convert to lower case to recognize the correct domain laterly */
+	for(p = LocalDomain ; *p ; p++)
+		*p = (char)tolower((int)*p);
+
+	/* we now have our hostname and can set it inside the global vars.
+	 * TODO: think if all of this would better be a runtime function
+	 * rgerhards, 2008-04-17
+	 */
+	glbl.SetLocalHostName(LocalHostName);
+	glbl.SetLocalDomain(LocalDomain);
+	glbl.GenerateLocalHostNameProperty(); /* must be redone after conf processing, FQDN setting may have changed */
+finalize_it:
+	RETiRet;
+}
+
+
 /* some support for command line option parsing. Any non-trivial options must be
  * buffered until the complete command line has been parsed. This is necessary to
  * prevent dependencies between the options. That, in turn, means we need to have
@@ -1565,7 +1684,7 @@ doGlblProcessInit(void)
 
 	if( !(Debug == DEBUG_FULL || NoFork) )
 	{
-		DBGPRINTF("Checking pidfile.\n");
+		DBGPRINTF("Checking pidfile '%s'.\n", PidFile);
 		if (!check_pid(PidFile))
 		{
 			memset(&sigAct, 0, sizeof (sigAct));
@@ -1573,6 +1692,9 @@ doGlblProcessInit(void)
 			sigAct.sa_handler = doexit;
 			sigaction(SIGTERM, &sigAct, NULL);
 
+			/* stop writing debug messages to stdout (if debugging is on) */
+			stddbg = -1;
+
 			if (fork()) {
 				/* Parent process
 				 */
@@ -1635,7 +1757,7 @@ doGlblProcessInit(void)
 	}
 
 	/* tuck my process id away */
-	DBGPRINTF("Writing pidfile %s.\n", PidFile);
+	DBGPRINTF("Writing pidfile '%s'.\n", PidFile);
 	if (!check_pid(PidFile))
 	{
 		if (!write_pid(PidFile))
@@ -1683,18 +1805,13 @@ doGlblProcessInit(void)
 int realMain(int argc, char **argv)
 {
 	rsRetVal localRet;
-	register uchar *p;
 	int ch;
-	struct hostent *hent;
 	extern int optind;
 	extern char *optarg;
 	int bEOptionWasGiven = 0;
 	int iHelperUOpt;
 	int bChDirRoot = 1; /* change the current working directory to "/"? */
 	char *arg;	/* for command line option processing */
-	uchar *LocalHostName;
-	uchar *LocalDomain;
-	uchar *LocalFQDNName;
 	char cwdbuf[128]; /* buffer to obtain/display current working directory */
 	DEFiRet;
 
@@ -1784,62 +1901,13 @@ int realMain(int argc, char **argv)
 
 	/* we need to create the inputName property (only once during our lifetime) */
 	CHKiRet(prop.Construct(&pInternalInputName));
-	CHKiRet(prop.SetString(pInternalInputName, UCHAR_CONSTANT("rsyslogd"), sizeof("rsyslgod") - 1));
+	CHKiRet(prop.SetString(pInternalInputName, UCHAR_CONSTANT("rsyslogd"), sizeof("rsyslogd") - 1));
 	CHKiRet(prop.ConstructFinalize(pInternalInputName));
 
-	CHKiRet(prop.Construct(&pLocalHostIP));
-	CHKiRet(prop.SetString(pLocalHostIP, UCHAR_CONSTANT("127.0.0.1"), sizeof("127.0.0.1") - 1));
-	CHKiRet(prop.ConstructFinalize(pLocalHostIP));
-
 	/* get our host and domain names - we need to do this early as we may emit
 	 * error log messages, which need the correct hostname. -- rgerhards, 2008-04-04
 	 */
-	net.getLocalHostname(&LocalFQDNName);
-	CHKmalloc(LocalHostName = (uchar*) strdup((char*)LocalFQDNName));
-	glbl.SetLocalFQDNName(LocalFQDNName); /* set the FQDN before we modify it */
-	if((p = (uchar*)strchr((char*)LocalHostName, '.'))) {
-		*p++ = '\0';
-		LocalDomain = p;
-	} else {
-		LocalDomain = (uchar*)"";
-
-		/* It's not clearly defined whether gethostname()
-		 * should return the simple hostname or the fqdn. A
-		 * good piece of software should be aware of both and
-		 * we want to distribute good software.  Joey
-		 *
-		 * Good software also always checks its return values...
-		 * If syslogd starts up before DNS is up & /etc/hosts
-		 * doesn't have LocalHostName listed, gethostbyname will
-                * return NULL.
-		 */
-		/* TODO: gethostbyname() is not thread-safe, but replacing it is
-		 * not urgent as we do not run on multiple threads here. rgerhards, 2007-09-25
-		 */
-		hent = gethostbyname((char*)LocalHostName);
-		if(hent) {
-			free(LocalHostName);
-			CHKmalloc(LocalHostName = (uchar*)strdup(hent->h_name));
-
-			if((p = (uchar*)strchr((char*)LocalHostName, '.')))
-			{
-				*p++ = '\0';
-				LocalDomain = p;
-			}
-		}
-	}
-
-	/* Convert to lower case to recognize the correct domain laterly */
-	for(p = LocalDomain ; *p ; p++)
-		*p = (char)tolower((int)*p);
-
-	/* we now have our hostname and can set it inside the global vars.
-	 * TODO: think if all of this would better be a runtime function
-	 * rgerhards, 2008-04-17
-	 */
-	glbl.SetLocalHostName(LocalHostName);
-	glbl.SetLocalDomain(LocalDomain);
-	glbl.GenerateLocalHostNameProperty(); /* must be redone after conf processing, FQDN setting may have changed */
+	queryLocalHostname();
 
 	/* initialize the objects */
 	if((iRet = modInitIminternal()) != RS_RET_OK) {
